@@ -8,7 +8,6 @@ use Fcntl ':flock';         # import LOCK_* constants
 use IO::Select;
 use Sys::Hostname;          # hostname()
 use Carp;
-use HTTP::Proxy::Engine;
 
 use strict;
 use vars qw( $VERSION $AUTOLOAD @METHODS
@@ -17,10 +16,11 @@ use vars qw( $VERSION $AUTOLOAD @METHODS
 require Exporter;
 @ISA    = qw(Exporter);
 @EXPORT = ();               # no export by default
-@EXPORT_OK = qw( NONE ERROR STATUS PROCESS SOCKET HEADERS FILTERS CONNECT ALL );
+@EXPORT_OK = qw( ERROR NONE    PROXY  STATUS PROCESS SOCKET HEADERS FILTERS
+                 DATA  CONNECT ENGINE ALL );
 %EXPORT_TAGS = ( log => [@EXPORT_OK] );    # only one tag
 
-$VERSION = '0.18';
+$VERSION = '0.20';
 
 my $CRLF = "\015\012";                     # "\r\n" is not portable
 
@@ -41,6 +41,10 @@ use constant CONNECT => 128;   # Data transmitted by the CONNECT method
 use constant ENGINE  => 256;   # Internal information from the Engine
 use constant ALL     => 511;   # All of the above
 
+# modules that need those constants to be defined
+use HTTP::Proxy::Engine;
+use HTTP::Proxy::FilterStack;
+
 # Methods we can forward
 my %METHODS;
 
@@ -311,7 +315,9 @@ sub serve_connections {
 
         # Got a request?
         unless ( defined $req ) {
-            $self->log( ERROR, "ERROR", "Getting request failed:", $conn->reason );
+            $self->log( ERROR, "ERROR",
+                "Getting request failed: " . $conn->reason )
+                if $conn->reason ne 'No more requests from this connection';
             return;
         }
         $self->log( STATUS, "REQUEST", $req->method . ' '
@@ -498,7 +504,7 @@ sub _send_response_headers {
 
     # correct headers
     $response->remove_header("Content-Length")
-      if $self->{body}{response}->active;
+      if $self->{body}{response}->will_modify();
     $response->header( Server => "HTTP::Proxy/$VERSION" )
       unless $response->header( 'Server' );
     $response->header( Date => time2str(time) )
@@ -509,7 +515,7 @@ sub _send_response_headers {
     else {
         my $code = $response->code;
         $conn->send_status_line( $code, $response->message,
-            $response->protocol );
+            $self->request()->protocol() );
         if ( $code =~ /^(1\d\d|[23]04)$/ ) {
 
             # make sure content is empty
@@ -717,7 +723,7 @@ sub log {
     my $level = shift;
     my $fh    = $self->logfh;
 
-    return unless $self->logmask & $level;
+    return unless $self->logmask & $level || $level == ERROR;
 
     my ( $prefix, $msg ) = ( @_, '' );
     my @lines = split /\n/, $msg;
@@ -728,150 +734,6 @@ sub log {
     flock( $fh, LOCK_UN );
 }
 
-#
-# This is an internal class to work more easily with filter stacks
-#
-# Here's a description of the class internals
-# - filters: the list of (sub, filter) pairs that match the message,
-#            and through which it must go
-# - current: the actual list of filters, which is computed during
-#            the first call to filter()
-# - buffers: the buffers associated with each (selected) filter
-# - body   : true if it's a HTTP::Proxy::BodyFilter stack
-#
-# a filter is actually a (matchsub, filterobj) pair
-# the matchsub is run againt the HTTP::Message object to find out if
-# the filter must be applied to it
-package HTTP::Proxy::FilterStack;
-
-use Carp;
-
-#
-# new( $isbody )
-# $isbody is true only for response-body filters stack
-sub new {
-    my $class = shift;
-    my $self  = {
-        body => shift || 0,
-        filters => [],
-        buffers => [],
-        current => undef,
-    };
-    $self->{type} = $self->{body} ? "HTTP::Proxy::BodyFilter"
-                                  : "HTTP::Proxy::HeaderFilter";
-    return bless $self, $class;
-}
-
-#
-# insert( $index, [ $matchsub, $filter ], ...)
-#
-sub insert {
-    my ( $self, $idx ) = ( shift, shift );
-    $_->[1]->isa( $self->{type} ) or croak("$_ is not a $self->{type}") for @_;
-    splice @{ $self->{filters} }, $idx, 0, @_;
-}
-
-#
-# remove( $index )
-#
-sub remove {
-    my ( $self, $idx ) = @_;
-    splice @{ $self->{filters} }, $idx, 1;
-}
-
-# 
-# push( [ $matchsub, $filter ], ... )
-# 
-sub push {
-    my $self = shift;
-    $_->[1]->isa( $self->{type} ) or croak("$_ is not a $self->{type}") for @_;
-    push @{ $self->{filters} }, @_;
-}
-
-sub all    { return @{ $_[0]->{filters} }; }
-sub active { return @{ $_[0]->{current} }; }
-
-#
-# select the filters that will be used on the message
-#
-sub select_filters {
-    my ($self, $message ) = @_;
-
-    # first time we're called this round
-    if ( not defined $self->{current} ) {
-
-        # select the filters that match
-        $self->{current} =
-          [ map { $_->[1] } grep { $_->[0]->() } @{ $self->{filters} } ];
-
-        # create the buffers
-        if ( $self->{body} ) {
-            $self->{buffers} = [ ( "" ) x @{ $self->{current} } ];
-            $self->{buffers} = [ \( @{ $self->{buffers} } ) ];
-        }
-
-        # start the filter if needed (and pass the message)
-        for ( @{ $self->{current} } ) {
-            if    ( $_->can('begin') ) { $_->begin( $message ); }
-            elsif ( $_->can('start') ) {
-                $_->proxy->log( HTTP::Proxy::ERROR, "DEPRECATION", "The start() filter method is *deprecated* and disappeared in 0.15!\nUse begin() in your filters instead!" );
-            }
-        }
-    }
-}
-
-#
-# the actual filtering is done here
-#
-sub filter {
-    my $self = shift;
-
-    # pass the body data through the filter
-    if ( $self->{body} ) {
-        my $i = 0;
-        my ( $data, $message, $protocol ) = @_;
-        for ( @{ $self->{current} } ) {
-            $$data = ${ $self->{buffers}[$i] } . $$data;
-            ${ $self->{buffers}[ $i ] } = "";
-            $_->filter( $data, $message, $protocol, $self->{buffers}[ $i++ ] );
-        }
-    }
-    else {
-        $_->filter(@_) for @{ $self->{current} };
-        $self->eod;
-    }
-}
-
-#
-# filter what remains in the buffers
-#
-sub filter_last {
-    my $self = shift;
-    return unless $self->{body};    # sanity check
-
-    my $i = 0;
-    my ( $data, $message, $protocol ) = @_;
-    for ( @{ $self->{current} } ) {
-        $$data = ${ $self->{buffers}[ $i ] } . $$data;
-        ${ $self->{buffers}[ $i++ ] } = "";
-        $_->filter( $data, $message, $protocol, undef );
-    }
-
-    # call the cleanup routine if needed
-    for ( @{ $self->{current} } ) { $_->end if $_->can('end'); }
-    
-    # clean up the mess for next time
-    $self->eod;
-}
-
-#
-# END OF DATA cleanup method
-#
-sub eod {
-    $_[0]->{buffers} = [];
-    $_[0]->{current} = undef;
-}
-
 1;
 
 __END__
