@@ -35,7 +35,7 @@ sub init {
     croak "status must contain only HTTP codes"
       if grep { !/^[12345]\d\d$/ } @{ $args{status} };
     croak "filename must be a code reference"
-      if defined $args{filename} && UNIVERSAL::isa($args{filename}, 'CODE');
+      if defined $args{filename} && !UNIVERSAL::isa( $args{filename}, 'CODE' );
 
     $self->{"_hpbf_save_$_"} = $args{$_}
       for qw( template no_host no_dirs cut_dirs prefix
@@ -157,6 +157,8 @@ sub end {
     }
 }
 
+sub will_modify { 0 }
+
 1;
 
 __END__
@@ -197,6 +199,9 @@ in the stack, the saved data can be more or less modified.
 
 This filter I<will> create directories if it needs to!
 
+I<Note:> Remember that the default C<mime> parameter for C<push_filter()>
+is C<text/*> and that you may need to change it for other MIME types.
+
 =head2 Constructor
 
 The constructor accepts quite a few options. Most of them control
@@ -369,6 +374,11 @@ Save all the data that goes through to the opened file.
 
 Close the file when the whole message body has been processed.
 
+=item will_modify()
+
+This method returns a I<false> value, thus indicating to the system
+that it will not modify data passing through.
+
 =back
 
 =head1 SEE ALSO
@@ -396,9 +406,12 @@ Thanks to Howard Jones for the inspiration and initial patch for the
 C<filename> option. Lucas Gonze provided a patch to make C<status>
 actually work.
 
+Thanks to Max Maischein for detecting a bug in the paramerter validation
+for C<filename>.
+
 =head1 COPYRIGHT
 
-Copyright 2004-2005, Philippe Bruhat.
+Copyright 2004-2006, Philippe Bruhat.
 
 =head1 LICENSE
 
